<?php

namespace Drupal\rest_api_access_token\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\rest_api_access_token\Authentication\Provider\AccessTokenProvider;
use Drupal\rest_api_access_token\Service\LoginService;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class AuthController extends ControllerBase
{
    /** @var AccessTokenProvider */
    protected $accessTokenProvider;

    /** @var LoginService */
    protected $loginService;

    /**
     * AuthController constructor.
     * @param AccessTokenProvider $accessTokenProvider
     * @param LoginService $loginService
     */
    public function __construct(AccessTokenProvider $accessTokenProvider, LoginService $loginService)
    {
        $this->accessTokenProvider = $accessTokenProvider;
        $this->loginService = $loginService;
    }

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container)
    {
        $controller = new static(
            $container->get('rest_api_access_token.authentication.access_token_provider'),
            $container->get('rest_api_access_token.authentication.login')
        );

        return $controller;
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws \Drupal\rest_api_access_token\Exception\AuthenticationException
     * @throws \Drupal\rest_api_access_token\Exception\TokenGeneratorException
     */
    public function tokenResponse(Request $request)
    {
        $data = json_decode($request->getContent(), true);

        $login = (string)($data['login'] ?? '');
        $password = (string)($data['password'] ?? '');

        $token = $this->loginService->login($login, $password);

        $response = [
            'token' => $token->getPublic(),
            'secret' => $token->getSecret(),
            'userId' => $token->getUserId(),
        ];

        return new JsonResponse($response, JsonResponse::HTTP_OK);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function logout(Request $request)
    {
        $publicToken = (string)$request->headers->get('X-AUTH-TOKEN');
        $status = $this->loginService->logout($publicToken);
        $response = [
            'loggedOut' => $status,
        ];
        return new JsonResponse($response, JsonResponse::HTTP_OK);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function logoutFromAllDevices(Request $request)
    {
        /** @var AccountProxyInterface $account */
        $account = \Drupal::currentUser();

        /** @var User $user */
        $user = $account->getAccount();
        $status = $this->loginService->logoutFromAllDevices($user);
        $response = [
            'loggedOut' => $status,
        ];
        return new JsonResponse($response, JsonResponse::HTTP_OK);
    }
}
