<?php

namespace Drupal\rest_api_access_token\EventSubscriber;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class CacheEndpointSubscriber implements EventSubscriberInterface
{
    const CACHE_KEY = 'rest_api_access_token_cache';

    /** @var ConfigFactoryInterface */
    protected $configFactory;

    /**
     * CacheEndpointSubscriber constructor.
     * @param ConfigFactoryInterface $configFactory
     */
    public function __construct(ConfigFactoryInterface $configFactory)
    {
        $this->configFactory = $configFactory;
    }

    /**
     * @param string $requestId
     * @param string $userToken
     * @param string $path
     * @return string
     */
    protected function getCacheKey(string $requestId, string $userToken, string $path)
    {
        return self::CACHE_KEY . ':' . md5($userToken) . ':' . str_replace('/', '.', $path) . ':' . $requestId;
    }

    /**
     * @param GetResponseEvent $event
     */
    public function onKernelRequest(GetResponseEvent $event)
    {
        if (!$event->isMasterRequest()) {
            return;
        }

        $request = $event->getRequest();
        $requestId = (string)$request->headers->get('REQUEST-ID');
        $publicToken = (string)$request->headers->get('X-AUTH-TOKEN');

        if ($this->configFactory->get('rest_api_access_token.config')->get('cache_endpoints') && !empty($requestId)) {
            $path = $request->getPathInfo();
            $cid = $this->getCacheKey($requestId, $publicToken, $path);
            if ($cache = \Drupal::cache()->get($cid)) {
                $cacheResponse = $cache->data;
                if ($cacheResponse instanceof Response) {
                    $cacheResponse->send();
                    exit;
                }
            }
        }
    }

    /**
     * @param FilterResponseEvent $event
     */
    public function onKernelResponse(FilterResponseEvent $event)
    {
        $response = $event->getResponse();
        $request = $event->getRequest();
        $requestId = (string)$request->headers->get('REQUEST-ID');
        $publicToken = (string)$request->headers->get('X-AUTH-TOKEN');

        if ($this->configFactory->get('rest_api_access_token.config')->get('cache_endpoints') && !empty($requestId)) {
            $path = $request->getPathInfo();
            $cid = $this->getCacheKey($requestId, $publicToken, $path);
            $currentTime = \Drupal::time()->getCurrentTime();
            $lifetime = (int)$this->configFactory->get('rest_api_access_token.config')->get('cache_endpoints_lifetime');
            $expire = $currentTime + $lifetime;
            if ($lifetime <= 0) {
                $expire = Cache::PERMANENT;
            }
            \Drupal::cache()->set($cid, $response, $expire);
        }
    }

    /**
     * {@inheritdoc}
     */
    public static function getSubscribedEvents()
    {
        $events[KernelEvents::REQUEST][] = ['onKernelRequest', 999];
        $events[KernelEvents::RESPONSE][] = ['onKernelResponse', -999];
        return $events;
    }
}
