<?php

namespace Drupal\rest_api_access_token\Repository;

use Drupal\Core\Database\Connection;
use Drupal\rest_api_access_token\Exception\TokenNotFoundException;
use Drupal\rest_api_access_token\Model\Token;
use Drupal\user\Entity\User;

class TokenRepository
{
    /** @var Connection */
    protected $connection;

    /**
     * TokenRepository constructor.
     * @param Connection $connection
     */
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }

    /**
     * @param string $token
     * @return Token
     * @throws TokenNotFoundException
     */
    public function getByPublicToken(string $token)
    {
        $tokenRaw = $this->connection
            ->select('rest_api_access_token', 't')
            ->fields('t')
            ->condition('public_token', $token)
            ->execute()
            ->fetchObject();

        if (empty($tokenRaw)) {
            throw new TokenNotFoundException($token);
        }

        return new Token($tokenRaw->public_token, $tokenRaw->secret, $tokenRaw->user_id);
    }

    /**
     * @param Token $accessToken
     * @return int
     * @throws \Exception
     */
    public function insert(Token $accessToken)
    {
        return (int)$this->connection
            ->insert('rest_api_access_token')
            ->fields([
                'user_id' => $accessToken->getUserId(),
                'public_token' => $accessToken->getPublic(),
                'secret' => $accessToken->getSecret(),
            ])
            ->execute();
    }

    /**
     * @param string $token
     * @return int
     */
    public function removeByPublicToken(string $token)
    {
        return $this->connection->delete('rest_api_access_token')
            ->condition('public_token', $token)
            ->execute();
    }

    /**
     * @param User $user
     * @return int
     */
    public function removeByUser(User $user)
    {
        return $this->connection->delete('rest_api_access_token')
            ->condition('user_id', $user->id())
            ->execute();
    }
}
