<?php

namespace Drupal\rest_api_access_token\Service;

use Drupal\rest_api_access_token\Exception\AuthenticationException;
use Drupal\rest_api_access_token\Exception\TokenGeneratorException;
use Drupal\rest_api_access_token\Exception\TokenNotFoundException;
use Drupal\rest_api_access_token\Repository\TokenRepository;
use Drupal\user\Entity\User;

class LoginService
{
    /** @var TokenGenerator */
    protected $tokenGenerator;

    /** @var TokenRepository */
    protected $tokenRepository;

    /**
     * LoginService constructor.
     * @param TokenGenerator $tokenGenerator
     * @param TokenRepository $tokenRepository
     */
    public function __construct(TokenGenerator $tokenGenerator, TokenRepository $tokenRepository)
    {
        $this->tokenGenerator = $tokenGenerator;
        $this->tokenRepository = $tokenRepository;
    }


    /**
     * @param string $login
     * @param string $password
     * @return \Drupal\rest_api_access_token\Model\Token
     * @throws AuthenticationException
     * @throws TokenGeneratorException
     * @throws \Exception
     */
    public function login(string $login, string $password)
    {
        if (empty($login) || empty($password)) {
            throw new AuthenticationException("Invalid user login or password");
        }

        // login by email
        $usersByEmail = \Drupal::entityTypeManager()->getStorage('user')
            ->loadByProperties(['mail' => $login]);

        /** @var User $userByEmail */
        $userByEmail = reset($usersByEmail);

        if ($userByEmail) {
            $login = $userByEmail->getUsername();
        }

        // Authenticate user
        $uid = (int)\Drupal::service('user.auth')->authenticate($login, $password);
        if ($uid <= 0) {
            throw new AuthenticationException("Invalid user login or password");
        }

        try {
            for ($i = 0; $i < 5; $i++) {
                $token = $this->tokenGenerator->execute($uid);
                $this->tokenRepository->getByPublicToken($token->getPublic());
                $token = false;
            }
        } catch (TokenNotFoundException $exception) {

        }

        if (empty($token)) {
            throw new TokenGeneratorException("Error during generate token");
        }

        $this->tokenRepository->insert($token);

        return $token;
    }

    /**
     * @param string $publicToken
     * @return bool
     */
    public function logout(string $publicToken)
    {
        return (bool)$this->tokenRepository->removeByPublicToken($publicToken);
    }

    /**
     * @param User $user
     * @return bool
     */
    public function logoutFromAllDevices(User $user)
    {
        return (bool)$this->tokenRepository->removeByUser($user);
    }
}
